package geniusweb.boa;

import java.util.HashMap;

import geniusweb.actions.Action;
import geniusweb.boa.acceptancestrategy.AcceptanceStrategy;
import geniusweb.boa.biddingstrategy.BiddingStrategy;
import geniusweb.inform.Settings;
import geniusweb.opponentmodel.OpponentModel;
import tudelft.utilities.logging.Reporter;

/**
 * Determines the next {@link Action} given our profile, the opponent models,
 * the acceptance strategy and BiddingStrategy.
 * <p>
 * The settings are assumed to contain parameters to set the BOA components:
 * <ul>
 * <li>"as" containing the full.class.path to the {@link AcceptanceStrategy}
 * clas,
 * <li>"os" containing the full.class.path to the   {@link OpponentModel} class
 * <li>"bs" containing the full.class.path to the   {@link BiddingStrategy}
 * class.
 * <li>Other parameters as needed by the components
 * </ul>
 * <h1>more info</h1> For more information, see: Baarslag T., Hindriks K.V.,
 * Hendrikx M., Dirkzwager A., Jonker C.M. Decoupling Negotiating Agents to
 * Explore the Space of Negotiation Strategies. Proceedings of The Fifth
 * International Workshop on Agent-based Complex Automated Negotiations (ACAN
 * 2012), 2012.
 * https://homepages.cwi.nl/~baarslag/pub/Decoupling_Negotiating_Agents_to_Explore_the_Space_of_Negotiation_Strategies_ACAN_2012.pdf
 * 
 */
public class BoaParty extends DefaultBoa {
	public BoaParty() {
		super();
	}

	public BoaParty(Reporter reporter) {
		super(reporter); // for debugging
	}

	@Override
	public String getDescription() {
		return "BOA party that is fully configurable through parameters. "
				+ "Takes parameters om=OpponentModel full.class.path, "
				+ "as=AcceptanceStrategy full.class.path "
				+ "and bs=BiddingStrategy full.class.path. ";
	}

	/**
	 * Overridable getter for the acceptance strategy. Override if you want to
	 * hard code a {@link BoaParty} behaviour
	 * 
	 * @param settings the {@link Settings}
	 * @return the {@link OpponentModel} class to use for modeling opponents
	 */
	@Override
	protected Class<? extends OpponentModel> getOpponentModel(Settings settings)
			throws InstantiationFailedException {
		HashMap<String, Object> parameters = settings.getParameters();
		return loadClass("om", OpponentModel.class, parameters);
	}

	/**
	 * Overridable getter for the acceptance strategy. Override if you want to
	 * hard code a {@link BoaParty} behaviour
	 * 
	 * @param settings the {@link Settings}
	 * @return the {@link BiddingStrategy} to use for determining the next
	 *         {@link Action}
	 */
	@Override
	protected BiddingStrategy getBiddingStrategy(Settings settings)
			throws InstantiationFailedException {
		try {
			return loadClass("bs", BiddingStrategy.class,
					settings.getParameters()).newInstance();
		} catch (InstantiationException | IllegalAccessException e) {
			throw new InstantiationFailedException(
					"Can't instantiate bidding strategy", e);
		}
	}

	/**
	 * Overridable getter for the acceptance strategy. Override if you want to
	 * hard code a {@link BoaParty} behaviour
	 * 
	 * @param settings the {@link Settings}
	 * @return the {@link AcceptanceStrategy} to use for determining if a bid is
	 *         acceptable
	 */
	@Override
	protected AcceptanceStrategy getAccceptanceStrategy(Settings settings)
			throws InstantiationFailedException {
		try {
			return loadClass("as", AcceptanceStrategy.class,
					settings.getParameters()).newInstance();
		} catch (InstantiationException | IllegalAccessException e) {
			throw new InstantiationFailedException(
					"Can't instantiate acceptance strategy", e);
		}
	}

	/**
	 * 
	 * @param <T>        the type of the class object to load
	 * @param paramname  the name of the parameter in parameters that contains
	 *                   the class reference, shoul dbe full.clsss.path
	 * @param clazz      the class object to load
	 * @param parameters the parameterlist
	 * @return object of T, loaded from the parameter with paramname
	 * @throws ClassNotFoundException
	 * @throws InstantiationFailedException
	 */
	@SuppressWarnings("unchecked")
	private <T> Class<T> loadClass(String paramname, Class<T> clazz,
			HashMap<String, Object> parameters)
			throws InstantiationFailedException {

		if (!parameters.containsKey(paramname))
			throw new InstantiationFailedException(
					"Can not load BOA component: the parameter " + paramname
							+ " is not specified");
		Class<?> c;
		try {
			c = Class.forName((String) parameters.get(paramname));
		} catch (ClassNotFoundException e) {
			throw new InstantiationFailedException("Unknown BOA component", e);
		}
		if (!clazz.isAssignableFrom(c))
			throw new InstantiationFailedException("Parameter " + paramname
					+ " requires an instance of  " + clazz + ", but got " + c);

		return (Class<T>) c;
	}

}
